/****************************************************************************
* e2Ligth4 - Light Control  for Rokr E2
*
* Autor: Carlos Aguilar aka "Kidscracker" kidscracker@gmail.com
*
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//#define EZX_H

#include "ZGui.h"
#include "zLanguage_Panel.h"
#include "zLights_Panel.h"
#include "zBT_Panel.h"
#include "zSound_Panel.h"
#include "zSDManager_Panel.h"

#define EZX_SYSTEM_CFG "/ezx_user/download/appwrite/setup/ezx_system.cfg"
#define EZX_SECURITYSETUP_CFG "/ezx_user/download/appwrite/ezx_securitysetup.cfg"
#define CMD_RESTART_PHONE "reboot -d -f -i"
using namespace std;

extern ZApplication *app;
extern LanguageConfig *lng;
extern ZConfig *cfg;

ZConfig *ezx_system;

ZGui::ZGui ( QWidget* parent, const char* name, QDir homePath, WFlags fl ) : 
	ZKbMainWidget ( ZHeader::MAINDISPLAY_HEADER, NULL, "ZMainWidget", 0 )
{ 
	setAppDir(homePath);

	tabLights = NULL;
	tabLanguage = NULL;
	tabBT= NULL;
	tabSound = NULL;
	tabSDManager = NULL;
	
	SoftKey = NULL;

	mnMainMenu = NULL;
	mnOtherOptions = NULL;
	mnResetCodes = NULL;

		
	cout<<".::: e2Ligth4 - Beta4:::."<<endl<<endl;
	cout<<"  Base Path : "<<getAppPath()<<endl;
  
	// read ezx_system.cfg and load to memory
	ezx_system = new ZConfig(EZX_SYSTEM_CFG);
	
	cout<<"Creating Controls..."<<endl;
	CreateControls( parent );

	cout<<"Loading Configs..."<<endl;
	LoadConfigs();
	cout<<"Applying Language Setting..."<<endl;
	cout<<"\t> Refreshing Controls Language..."<<endl;
	refreshControlsLanguage();
	cout<<"\t> Updating Control Values..."<<endl;
	updateControlValues(true);

	// timer for checking control values
	startTimer(200);

	( ( ZApplication* ) qApp )->showMainWidget ( this );
	return;
}

ZGui::~ZGui()
{
//	cout<<"ZGui::~ZGui - Entry"<<endl;
//	cout<<"ZGui::~ZGui - Exit"<<endl;
}

void ZGui::CreateControls(QWidget* parent)
{
	// Set Title
	setMainWidgetTitle(QString::fromUtf8("e2Light4 - Beta4"));

	// Default Layaout is Vertical
	QVBoxLayout* layout = getVBoxLayout();

	// Create TabWidget
	tabWidget = new ZNavTabWidget(0, false, true, this, "tabWidget",(ZSkinService::WidgetClsID)40);
		tabWidget->resize(240,100);
		
	layout->addWidget(tabWidget);

	connect(tabWidget, SIGNAL(currentChanged(QWidget*)), this, SLOT(pageChanged(QWidget*)));
	
//	PANEL 1 - Lights Control
	QPixmap pm( getAppDir().absFilePath("img/e2light4.png") );
	// Add Tab for Panel 1 - Light Control
	tabWidget->addTab(getLightsTab(), QIconSet(pm), QString(""));

//	PANEL 2 - BT Control
	pm.load( getAppDir().absFilePath("img/bluetooth.png") );
	// Add Tab for Panel 2 - BT Control
	tabWidget->addTab(getBTTab(), QIconSet(pm), QString(""));

//	PANEL 3 - Sound Control
	pm.load( getAppDir().absFilePath("img/sound.png") );
	// Add Tab for Panel 3 - Sound Control
	tabWidget->addTab(getSoundTab(), QIconSet(pm), QString(""));

//	PANEL 4 - SDManager
	pm.load( getAppDir().absFilePath("img/sdmanager.png") );
	// Add Tab for Panel 4 - SD Manager
	tabWidget->addTab(getSDManagerTab(), QIconSet(pm), QString(""));
	
//	PANEL 5 - Language Selector
	pm.load( getAppDir().absFilePath("img/config.png") );
	// Add Tab for Panel 5 - Language Selector
	tabWidget->addTab(getLanguageTab(), QIconSet(pm), QString(""));
	
	//setContentWidget ( tabWidget );

//	Set SoftKeys
	setSoftKey( getSoftKey() );

	resize(240,320);

}

ZSoftKey* ZGui::getSoftKey()
{
	if (SoftKey!=NULL)
		return SoftKey;
		
	SoftKey = new ZSoftKey(NULL , this , this);

	SoftKey->setOptMenu(ZSoftKey::LEFT, getmnMainMenu() );

	SoftKey->setClickedSlot(ZSoftKey::RIGHT, this, SLOT(slotAbout()) );
	//menu->connectItem(0, this, SLOT(set()));

	connect( getmnMainMenu(), SIGNAL(aboutToHide()), this, SLOT( slotMenu() ) );

	return SoftKey;
}

ZOptionsMenu* ZGui::getmnMainMenu()
{
	if (mnMainMenu!=NULL)
		return mnMainMenu;

	QRect rect;
	mnMainMenu = new ZOptionsMenu(rect, SoftKey, "MENU" , 0 ,(ZSkinService::WidgetClsID)2 );
		mnMainMenu->insertItem("", getmnOtherOptions(), NULL , true , 1 , -1 );
		mnMainMenu->insertItem("", NULL , NULL, true , 0, -1 );

	mnMainMenu->connectItem(0, this, SLOT(quit()));	
	return mnMainMenu;
}
ZOptionsMenu* ZGui::getmnOtherOptions()
{
	if (mnOtherOptions!=NULL)
		return mnOtherOptions;

	QRect rect;
	mnOtherOptions = new ZOptionsMenu(rect, SoftKey, "OTHER_OPTIONS" , 0 ,(ZSkinService::WidgetClsID)2 );
		mnOtherOptions->insertItem("", getmnResetCodes() , NULL, true , 0, -1 );
		mnOtherOptions->insertItem("Restart Phone", NULL , NULL, true , 1, -1 );
	
	mnOtherOptions->connectItem(1, this, SLOT(slotRestartPhone()));	
	return mnOtherOptions;
}

ZOptionsMenu* ZGui::getmnResetCodes()
{
	if (mnResetCodes!=NULL)
		return mnResetCodes;

	QRect rect;
	mnResetCodes = new ZOptionsMenu(rect, SoftKey, "RESET_CODE" , 0 ,(ZSkinService::WidgetClsID)2 );
		mnResetCodes->insertItem("", NULL, NULL , true , 1 , -1 );
		mnResetCodes->insertItem("", NULL , NULL, true , 0, -1 );

	mnResetCodes->connectItem(1, this, SLOT(slotResetSecurityUnlockCode()));	// Codigo de Bloqueo
	mnResetCodes->connectItem(0, this, SLOT(slotResetSecuritySecurityCode()));	// Codigo de Seguridad
	
	return mnResetCodes;
}

void ZGui::LoadConfigs()
{
	// Directory where language files are stored
	string lng_file_rel = "lang/";
	
	// Config file path
	QString cfg_file = getAppDir().absFilePath("config.cfg");

	cout<<"Config File: "<<cfg_file<<endl;

	// Create ConfigFile Object for App configuration
	cfg = new ZConfig( cfg_file );

	// If Group doesn't exist, create sections needed and perform all needed test to autoconfigure
	if (!cfg->groupExists("E2LIGHT"))
	{
		cout<<"Generating configuration file"<<endl;
		cout<<"Autodetecting language ... ";
		if (!AutodetecLanguage(QString(lng_file_rel.c_str())) )
		{
			cout<<"Unable to found adecuated Language File"<<endl;
			cout<<"Setting ENGLISH as Default ...";
			cfg->writeEntry("E2LIGHT","LANGUAGE_FILE","english.cfg");
			cout<<"[DONE]"<<endl;
		}

	}

	lng_file_rel.append( cfg->readEntry("E2LIGHT","LANGUAGE_FILE", "LANGUAGE_FILE") );

	QString lng_file = getAppDir().absFilePath( lng_file_rel.c_str() );
	cout<<"Language File : "<<lng_file<<endl;
	lng = new LanguageConfig( lng_file );

}

bool ZGui::GetFileList(QDir &dir, QStringList & fl)
{
    int i, nCount = dir.count();
	QString current_lang_file( cfg->readEntry("E2LIGHT","LANGUAGE_FILE","english.cfg") );
	
    for (i = 0; i < nCount; i ++)
    {
        fl.append(QString(dir[i]));
//		cout<<"\tAgregando archivo "<<dir[i]<<endl;
		if (current_lang_file.compare(dir[i])==0)
		{
			currentLangSelect=i;
		}
    }
    return true;
}

void ZGui::Fill_LanguageListBox()
{
	ZConfig* tmp;
	QPixmap* pm = new QPixmap();
	QString tmp_lng_Short;
	QString lng_path;
	QString lng_img_path;
	
	slListFiles = QStringList();

    QString sDefaultDir = getAppDir().absFilePath("lang/");
	cout<<"Exploring language directory "<<sDefaultDir<<endl;
    QString sDefaultExt = QString("*.cfg");
    int nFilterSpec = QDir::Files;
	
	QDir dir(sDefaultDir, sDefaultExt, QDir::Name | QDir::IgnoreCase, nFilterSpec );
	GetFileList(dir, slListFiles);

	ZListBoxItem* eitem;

	getLanguageTab()->getlstIdioma()->clear();
	
    for ( QStringList::Iterator it = slListFiles.begin(); it != slListFiles.end(); ++it ) {
		eitem= new ZListBoxItem ( getLanguageTab()->getlstIdioma(), QString ( "%R16%I%M" ) );

		lng_path = getAppDir().absFilePath("lang/") + QString(*it);
		// Load Language File
		tmp = new ZConfig( lng_path );

		// Get its Short Description to form img path and name
		tmp_lng_Short = tmp->readEntry("LANG_FILE_INFORMATION", "SHORT");
		lng_img_path = getAppDir().absFilePath("img/") + tmp_lng_Short + ".png";
		// Load Image for the Language File
		pm->load( lng_img_path );
		//Create item
		eitem->appendSubItem ( 2, tmp->readEntry("LANG_FILE_INFORMATION","LANGUAGE", ""), false, pm);
		// Insert in the ListBox
		getLanguageTab()->getlstIdioma()->insertItem(eitem, -1, true);
    }

	getLanguageTab()->getlstIdioma()->checkItem(currentLangSelect,true);
	getLanguageTab()->getlstIdioma()->setCurrentItem(currentLangSelect);

}

void ZGui::updateControlValues(bool force)
{
	int selectOpcion = getLightsTab()->getcbOpcion()->currentItem();
	int index=-1;
	int max_value[]={120,120,20,15};
	char* strOptionName[] = {"ScreenSaverTimeout","DiaplyTimeout","BacklightTimeout","KeypadBacklightTimeout"};
	QString valor;
	QString strNombreOpcion("");
	if (currentOption!=selectOpcion | force)
	{
		valor.setNum( getLightsTab()->getnmValorOpcion()->getValue() );

		if (currentOption<4)
			ezx_system->writeEntry("SYS_SYSTEM_SET", strOptionName[currentOption], valor);

		currentOption=selectOpcion;

		getLightsTab()->getnmValorOpcion()->setMinValue(0);
		getLightsTab()->getnmValorOpcion()->setMaxValue(max_value[currentOption]);

		if (currentOption<4)
			index = ezx_system->readNumEntry("SYS_SYSTEM_SET", strOptionName[currentOption], 0);

		getLightsTab()->getnmValorOpcion()->setValue(index);
	}
}

void ZGui::setAppDir(QDir dr)
{
	Home_Dir = dr;
	return;
}

QDir ZGui::getAppDir()
{
	if (!Home_Dir.exists())
	{
		QString ProgDir = QString ( qApp->argv() [0] ) ;
		int i = ProgDir.findRev ( "/" );
		ProgDir.remove ( i+1, ProgDir.length() - i );
		Home_Dir = QDir(ProgDir);
	}
	return Home_Dir;
}

QString ZGui::getAppPath()
{
	if (Home_Path.isEmpty())
		Home_Path = getAppDir().path();
	return Home_Path;
}

/* Autodetect the language using ezx_system.cfg as guide */
bool ZGui::AutodetecLanguage(const QString& lng_path = QString())
{
	int i, nCount;

    // Absolute Paht to Language Files
	QString sDefaultDir = getAppDir().absFilePath(lng_path);
    QString sDefaultExt = QString("*.cfg");
    int nFilterSpec = QDir::Files;
	// QDir object for Language Files Filter
	QDir dir(sDefaultDir, sDefaultExt, QDir::Name | QDir::IgnoreCase, nFilterSpec );

	nCount = dir.count();
	
	// Short Name of currente LanguageType
	string lng_Short = ezx_system->readEntry("SYS_SYSTEM_SET","LanguageType","EN").ascii() ;

	cout<<" LanguageType : '"<<lng_Short<<"'"<<endl;

	// Temporary ConfigFile object, used while checking for compatible language file
	ZConfig* tmp;
	string tmp_lng_Short;
	
    for (i = 0; i < nCount; i ++)
    {
		// Load Language File
		tmp = new ZConfig( dir.absFilePath(dir[i]) );
		// Get its Short Description
		tmp_lng_Short = tmp->readEntry("LANG_FILE_INFORMATION","SHORT","").ascii();
		
		cout<<"\tTesting : "<<tmp_lng_Short<<endl;
		
		// Check if its the exact one or at least a compatible one
		if (tmp_lng_Short.compare(lng_Short)==0 || 
			tmp_lng_Short.compare(lng_Short.substr(0,lng_Short.find_first_of("-")))==0 )
		{
			// Set in the Config File
			cfg->writeEntry("E2LIGHT","LANGUAGE_FILE", dir[i] );
			cout<<"[DONE]"<<endl;

			cout<<"Language set to "<<tmp->readEntry("LANG_FILE_INFORMATION","LANGUAGE", "")<<endl;
			delete tmp;
			return true;
		}
		delete tmp;
    }	
	return false;
}

void ZGui::refreshControlsLanguage()
{

	// BT Tab Labels and Values
	getLightsTab()->slotLanguageChanged(false);
	// BT Tab Labels and Values
	getBTTab()->slotLanguageChanged(false);
	// Sound Tab Labels and Values
	getSoundTab()->slotLanguageChanged(false);
	// Language Panel
	getLanguageTab()->slotLanguageChanged(false);
	
	QPixmap* pm  = new QPixmap( getAppDir().absFilePath("img/config.png") );
	getmnMainMenu()->changeItem(1,"  " +lng->getText("OTHER_OPTIONS") , pm );

	pm->load( getAppDir().absFilePath("img/exit.png") );
	getmnMainMenu()->changeItem(0,"  " + lng->getText("EXIT"), pm);

	getSoftKey()->setText(ZSoftKey::LEFT, lng->getText("MENU"), (ZSoftKey::TEXT_PRIORITY)0);
	getSoftKey()->setText(ZSoftKey::RIGHT, lng->getText("ABOUT"), (ZSoftKey::TEXT_PRIORITY)0);

	pm->load( getAppDir().absFilePath("img/resetcode.png") );
	getmnOtherOptions()->changeItem(0,"  " + lng->getText("RESET_CODE"), pm);
		getmnResetCodes()->changeItem(1,"  " + lng->getText("RESET_UNLOCKCODE"), pm);
		getmnResetCodes()->changeItem(0,"  " + lng->getText("RESET_SECURITYCODE"), pm);
	
	pm->load( getAppDir().absFilePath("img/restart.png") );
		getmnOtherOptions()->changeItem(1,"  " + lng->getText("RESTART_PHONE"), pm);
	
	Fill_LanguageListBox();
}

zLights_Panel* ZGui::getLightsTab()
{
	if (tabLights)
		return tabLights;

	tabLights = new zLights_Panel( this );
	return tabLights;
}

zLanguage_Panel* ZGui::getLanguageTab()
{
	if (tabLanguage!=NULL)
		return tabLanguage;

	tabLanguage = new zLanguage_Panel( this );
	return tabLanguage;
}

zBT_Panel* ZGui::getBTTab()
{
	if (tabBT)
		return tabBT;

	tabBT = new zBT_Panel( this );
	return tabBT;
}

zSound_Panel* ZGui::getSoundTab()
{
	if (tabSound)
		return tabSound;

	tabSound = new zSound_Panel( this );
	return tabSound;
}

zSDManager_Panel* ZGui::getSDManagerTab()
{
	if (tabSDManager)
		return tabSDManager;

	tabSDManager = new zSDManager_Panel( this );
	return tabSDManager;
}

/*	SLOTS	*/
void ZGui::timerEvent(QTimerEvent* e)
{
	updateControlValues(false);
}

void ZGui::slotMenu()
{
	getSoftKey()->setText(ZSoftKey::LEFT, lng->getText("MENU"), (ZSoftKey::TEXT_PRIORITY)0);
}

void ZGui::quit()
{
	pdlg = new ZProgressDlg(lng->getText("EXITING"),  "", 100, 220);

    pdlg->setInstructText(lng->getText("SAVING_CHANGES"));

    if (pdlg)
    {
        pdlg->setAutoClose(true);
        pdlg->setProgress(0);
		pdlg->setProgressTotalStep(3);
        pdlg->show();
        QTimer::singleShot(100, this, SLOT(slotProgressDlgTimer()));
        connect(pdlg, SIGNAL(cancelled()), this, SLOT(slotProgressDlgCancelled()));
    }
}


void ZGui::slotProgressDlgTimer()
{
	if ((pdlg->getProgress() < pdlg->getProgressTotalStep()) && pdlg->isVisible())
	{
		switch(pdlg->getProgress())
		{
			case 0:
				updateControlValues(true);
				break;
			case 1:
				if ( ezx_system->changed() )
					ezx_system->flush();
				break;
			case 2:
				if ( cfg->changed() )
					cfg->flush();
				break;
		}
		pdlg->setProgress(pdlg->getProgress() + 1);
		QTimer::singleShot(4 * 10, this, SLOT(slotProgressDlgTimer()));
	}else if (pdlg->getProgress() < pdlg->getProgressTotalStep())
	{
		app->exit();
	}
	return;
}
void ZGui::slotProgressDlgCancelled()
{
}

void ZGui::slotAbout()
{
	QString msg("E2Light v0.4 Beta3\nby Carlos Aguilar aka Kidscracker\n");
	msg.append("<kidscracker@gmail.com>\n\nThanks to lasly and blackhawk for their SDK.\n");
	ZMessageDlg* about = new ZMessageDlg(lng->getText("ABOUT"), msg, (ZMessageDlg::MessageDlgType)2, 0, this, "about", true, 0);
	about->show();
	return;
}

void ZGui::slotLanguageListSelected ( int index ) 
{
	if(	currentLangSelect != index )
	{
		currentLangSelect=index;
		cout<<"Index : "<<index<<endl;
		
		cout<<" Language_File : "<<slListFiles[currentLangSelect]<<endl;

		cfg->writeEntry( "E2LIGHT", "LANGUAGE_FILE", slListFiles[currentLangSelect] );
		string lng_file_rel = "lang/";
		QString lng_file = getAppDir().absFilePath( lng_file_rel.c_str() );
		lng_file.append( cfg->readEntry("E2LIGHT","LANGUAGE_FILE") );
		lng = new LanguageConfig( lng_file );
		
		refreshControlsLanguage();
		updateControlValues(true);
	}
}


void ZGui::slotResetSecurityUnlockCode()
{
	ZMessageDlg *dlg = new ZMessageDlg(lng->getText("ATENTION"), lng->getText("RESET_UNLOCKCODE_1234"), ZMessageDlg::TypeConfirm, 0, this, "dlg", true, 0);
	if (dlg->exec())
	{
		ZConfig* ezx_securitysetup = new ZConfig(EZX_SECURITYSETUP_CFG);
			ezx_securitysetup->writeEntry( "SYS_SECURITY", "SecurityUnlockCode","81dc9bdb52d04dc20036dbd8313ed055");
			ezx_securitysetup->flush();
	}
}
void ZGui::slotResetSecuritySecurityCode()
{
	ZMessageDlg *dlg = new ZMessageDlg(lng->getText("ATENTION"), lng->getText("RESET_SECURITYCODE_000000"), ZMessageDlg::TypeConfirm, 0, this, "dlg", true, 0);
	if (dlg->exec())
	{
		ZConfig* ezx_securitysetup = new ZConfig(EZX_SECURITYSETUP_CFG);
			ezx_securitysetup->writeEntry( "SYS_SECURITY", "SecuritySecurityCode","670b14728ad9902aecba32e22fa4f6bd");
			ezx_securitysetup->flush();
	}
}

void ZGui::slotRestartPhone()
{
	ZMessageDlg *dlg = new ZMessageDlg(lng->getText("ATENTION"), lng->getText("RESTART_PHONE") + "?", ZMessageDlg::TypeConfirm, 0, this, "dlg", true, 0);
	if (dlg->exec())
	{
		system(CMD_RESTART_PHONE);
		quit();
	}
}

void ZGui::tab1Activated (int index )
{
	cout<<"\tActivated en List1: index = "<<index<<endl;
}

void ZGui::pageChanged(QWidget* w)
{
	cout<<((zBase_Panel*)w)->getSoftKey()<<endl;
	cout<<"\tcurrentChanged en tabWidget : index = "<<tabWidget->currentPageIndex()<<endl;
	
}

void ZGui::slotOptionChange(int index)
{
	cout<<"Changed "<<index<<endl;
	updateControlValues(false);
}

void ZGui::keyPressEvent(QKeyEvent* event)
{
	cout<<event->key()<<endl;
	switch(event->key())
	{
		case EZX_KEY_END:
			ZMessageDlg *dlg = new ZMessageDlg(lng->getText("ATENTION"), lng->getText("CLOSE_APPLICATION"), ZMessageDlg::TypeConfirm, 0, this, "dlg", true, 0);
			if (dlg->exec())
			{
				quit();
			} 
			delete dlg;
			event->accept();
			return;
	}
	event->ignore();
	return;
}
