#ifndef __BT_STRUCTS_H
#define __BT_STRUCTS_H

#define MBT_MESSAGE_CLASS_CTRL      0x11 << 8
#define MBT_MSG_CTRL_POWER_STATUS_IND          (MBT_MESSAGE_CLASS_CTRL | 0x10)
#define MBT_MSG_CTRL_SET_DISCOVERABLE_MODE_CFM (MBT_MESSAGE_CLASS_CTRL | 0x11)
#define MBT_MSG_CTRL_DISCOVERABLE_STATUS_IND   (MBT_MESSAGE_CLASS_CTRL | 0x12)
#define MBT_MSG_CTRL_SET_LOCAL_COD_CFM         (MBT_MESSAGE_CLASS_CTRL | 0x13)

typedef enum {
    MBT_RESULT_SUCCESS = 0,        
    MBT_RESULT_FAIL,               
    MBT_RESULT_ERROR,              
    MBT_RESULT_NO_MEMORY,          
    MBT_RESULT_PEER_CLOSE,         
    MBT_RESULT_MSG_UNCOMPLETED,    
    MBT_RESULT_SOCKBUF_EMPTY,
    MBT_RESULT_INVALID_FD,
    MBT_RESULT_MSGQUEUE_FULL,
    MBT_RESULT_CONGESTED,
    MBT_RESULT_CALL_ERROR,
    MBT_RESULT_POWERED,            
    MBT_RESULT_POWERING,           
    MBT_RESULT_LIST_FULL,
    MBT_RESULT_MAX
} MBT_RESULT_T;

typedef int MBT_BOOL;
typedef signed char MBT_S8;
typedef signed int MBT_S32;

typedef unsigned int MBT_U32;
typedef unsigned short MBT_U16;
typedef unsigned char MBT_U8;

typedef MBT_U8 MBT_BLUETOOTH_ADDRESS_T[6];

typedef MBT_U32 MBT_PROFILE_TYPE_T;
typedef MBT_U16 MBT_PRIMITIVE_T;

typedef MBT_U8 MBT_MODULE_ID_T;

#define MBT_TRUE        1
#define MBT_FALSE       0


#define MBT_PROFILE_TYPE_ALL        0xFFFFFFFF

#define MBT_LOCAL_DEVICE_NAME_LEN 64

typedef struct
{
    MBT_MODULE_ID_T sender;          
    MBT_MODULE_ID_T receiver;        
    MBT_PRIMITIVE_T primitive;       
} MBT_MESSAGE_HEADER_T;

typedef enum
{
    MBT_DISCOVERABLE_MODE_OFF,       
    MBT_DISCOVERABLE_MODE_GIAC,      
    MBT_DISCOVERABLE_MODE_LIAC       
} MBT_DISCOVERABLE_MODE_T;

typedef enum
{
    MBT_POWER_ON_PARA_AUTO_POWER_OFF,
    MBT_POWER_ON_PARA_AUTO_POWER_OFF_TIME,
    MBT_POWER_ON_PARA_TEST_MODE,
    MBT_POWER_ON_PARA_SLEEP_MODE,
    MBT_POWER_ON_PARA_BAUD_RATE,
    MBT_POWER_ON_PARA_LOG_SWITCH,
    MBT_POWER_ON_PARA_FILE_LOG,
    MBT_POWER_ON_PARA_SLEEP_FOREVER_MODE,
    MBT_POWER_ON_PARA_DEVICE_NAME,

    MBT_POWER_ON_PARA_NUMBER
} MBT_POWER_ON_OPTION_TYPE_T;

typedef struct
{
    MBT_POWER_ON_OPTION_TYPE_T optionId;
    union
    {
        MBT_U8 devName[MBT_LOCAL_DEVICE_NAME_LEN + 1];
        MBT_S32 optionValue;
    };
} MBT_POWER_ON_OPTION_ITEM_T;

typedef struct
{
    MBT_MESSAGE_HEADER_T header;              
    MBT_BOOL status;                          
} MBT_CTRL_POWER_STATUS_IND_T;

typedef struct
{
    MBT_MESSAGE_HEADER_T header;              
    MBT_DISCOVERABLE_MODE_T mode;             
} MBT_CTRL_DISCOVERABLE_STATUS_IND_T;

typedef enum
{
    MBT_CTRL_SET_DISCOVERABLE_OK,            
    MBT_CTRL_SET_DISCOVERABLE_FAIL,          
    MBT_CTRL_SET_DISCOVERABLE_NOT_SUPPORT,
    MBT_CTRL_SET_DISCOVERABLE_REJECT,
    MBT_CTRL_SET_DISCOVERABLE_TIMEOUT,       
    MBT_CTRL_SET_DISCOVERABLE_STACK_BUSY     
} MBT_CTRL_SET_DISCOVERABLE_RESULT_T;

typedef struct
{
    MBT_MESSAGE_HEADER_T header;             
    MBT_DISCOVERABLE_MODE_T mode;            
    MBT_CTRL_SET_DISCOVERABLE_RESULT_T result;
} MBT_CTRL_SET_DISCOVERABLE_MODE_CFM_T;

extern "C" {
    int BT_PowerOn(void);
	int  UTIL_GetBtStatus();
	void UTIL_SetBtStatus(int s);

	extern MBT_S32 MBTAPI_Connect(MBT_PROFILE_TYPE_T proType, MBT_PRIMITIVE_T *pMsgId, MBT_U16 msgNum);

	extern MBT_BOOL MBTAPI_CTRL_CheckServerRunning(void);
	
	extern MBT_RESULT_T MBTAPI_CTRL_PowerOff(void);
	extern MBT_RESULT_T MBTAPI_CTRL_PowerOn(MBT_POWER_ON_OPTION_ITEM_T *optionList, MBT_U8 optionNum);
	extern MBT_RESULT_T MBTAPI_CTRL_SetDiscoverableMode(MBT_DISCOVERABLE_MODE_T mode, MBT_U16 second);
	extern MBT_RESULT_T MBTAPI_CTRL_GetDiscoverableMode(MBT_DISCOVERABLE_MODE_T *mode);
	extern MBT_RESULT_T MBTAPI_ReadAndMallocMsg(MBT_S32 fd, MBT_MESSAGE_HEADER_T **msg);

}

#endif
