/****************************************************************************
* e2Ligth4 - Light Control  for Rokr E2
*
* Autor: Carlos Aguilar aka "Kidscracker" kidscracker@gmail.com
*
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//#define EZX_H

#include "zBT_Panel.h"
#include "ZGui.h"

#define EZX_BLUETOOTH_CFG "/ezx_user/download/appwrite/setup/ezx_bluetooth.cfg"
/*
[Bluetooth Setup]
AutoPowerOffTimeout = 5
BluetoothPower = 1
BluetoothPowerStatusBeforeEnterAirPlaneMode = 0
DiscoverablePeriod = 2
defaultLocalDevName = MotoROKR ZINE2
defaultStorageLocation = 1
*/
using namespace std;

extern LanguageConfig *lng;
ZConfig* cfg_bt;
MBT_S32 socketFd = -1;

static MBT_PRIMITIVE_T msgList[] =
{
    MBT_MSG_CTRL_POWER_STATUS_IND,
    MBT_MSG_CTRL_SET_LOCAL_COD_CFM,
    MBT_MSG_CTRL_SET_DISCOVERABLE_MODE_CFM,
    MBT_MSG_CTRL_DISCOVERABLE_STATUS_IND
};
static MBT_PROFILE_TYPE_T proType = MBT_PROFILE_TYPE_ALL;
static MBT_BOOL powerStatus = MBT_FALSE;

static void SignalHandler(int sig)
{
    if (SIGALRM == sig)
    {
        if (powerStatus == MBT_FALSE)
        {
            fprintf(stderr,"Timeout to power on bluetooth, quit....\n");
            exit(-1);
        }
    }
}

MBT_BOOL ActivateBluetooth()
{
    MBT_RESULT_T powerRet = MBT_RESULT_POWERING;
    MBT_CTRL_POWER_STATUS_IND_T *powerStatusInd;

	//	Check whether bluetooth powered or not.
    if (MBTAPI_CTRL_CheckServerRunning())
    {
        fprintf(stderr,"Bluetooth has powered.\n");
        fprintf(stderr,"Establishing the socket connection ...\n");
        if ((socketFd = MBTAPI_Connect(proType, msgList, sizeof(msgList) / sizeof(MBT_PRIMITIVE_T))) <= 0)
        {
            fprintf(stderr,"Failed to establish the socket connection!\n");
            return MBT_FALSE;
        }
        else
        {
            fprintf(stderr,"Successfully establish the socket connection.\n");
            return MBT_TRUE;
        }
    }

	//	Power on bluetooth
	fprintf(stderr,"Bluetooth Server isn't RUNNING \n");
    fprintf(stderr,"Powering the bluetooth ...\n");
    powerRet = MBTAPI_CTRL_PowerOn(NULL, 0);

    if (powerRet == MBT_RESULT_FAIL)
    {
        fprintf(stderr,"Failed to power the bluetooth!\n");
        return MBT_FALSE;
   	}
    else if (powerRet == MBT_RESULT_POWERED)
   	{

		//	Establish socket connections
        socketFd = MBTAPI_Connect(proType, msgList, sizeof(msgList) / sizeof(MBT_PRIMITIVE_T));
        fprintf(stderr,"Successfully establish the socket connection.\n");
        return MBT_TRUE;
    }
    else if (powerRet == MBT_RESULT_POWERING)
   	{
		//	Power timeout timer
		fprintf(stderr,"Establishing socket connection with MBTAPI...\n");

        alarm(25);
        signal(SIGALRM, SignalHandler);

        while (socketFd == -1)
        {
            socketFd = MBTAPI_Connect(proType, msgList, sizeof(msgList) / sizeof(MBT_PRIMITIVE_T));
        }
		
        fprintf(stderr,"Successfully establish the socket connection.\n");

        while (MBTAPI_ReadAndMallocMsg(socketFd, (MBT_MESSAGE_HEADER_T **)&powerStatusInd) != MBT_RESULT_SUCCESS);
        powerStatus = powerStatusInd->status;
        free(powerStatusInd);

        if (powerStatus == MBT_FALSE)
        {
            fprintf(stderr,"Failed to power on the bluetooth!\n");
        }
        else
        {
            fprintf(stderr,"Successfully power on the bluetooth.\n");
        }

        return powerStatus;
    }else{
		fprintf(stderr,"MBTAPI_CTRL_PowerOn := 0x%08X\n",powerRet);
	}

    return MBT_FALSE;
}

zBT_Panel::zBT_Panel ( ZGui * _parent) : 
	zBase_Panel( (QWidget*)_parent, "tabPage_BT")
{
	// Set parent QWidget (ZGui in this case)
	parent = _parent;

	lblEstado = NULL;
	cbEstados = NULL;
	tabSPanel=NULL;
	lblMensaje=NULL;
	lblTiempoVisible=NULL;
	nmTiempoVisible=NULL;
	lblTiempoUnidad=NULL;
	pdlg = NULL;

	currentEstado = -1;
	// Set ContentWidget
	setContentWidget( getSPanel() );
	

	cfg_bt = new ZConfig(EZX_BLUETOOTH_CFG);

	tabSPanel->addChild( getlblEstado(), 0, 10);
	tabSPanel->addChild( getcbEstados(), 5, 40);

	tabSPanel->addChild( getlblTiempoVisible(), 0, 110);
	tabSPanel->addChild( getnmTiempoVisible(), 90, 90);
	
	if (!UTIL_GetBtStatus())
	{
		getcbEstados()->setEnabled(false);
		//tabSPanel->addChild( getlblMensaje() , 10, 80);
	}else{
		// If BT is Active just verify it and create a socket connection with  BT Server
		ActivateBluetooth();
	}

	getnmTiempoVisible()->setValue( cfg_bt->readNumEntry("Bluetooth Setup", "DiscoverablePeriod", 1) );
 
	// timer for checking control values
	startTimer(200);
	return;
}

zBT_Panel::~zBT_Panel()
{
	killTimers();
	if ( cfg_bt->changed() )
		cfg_bt->flush();
	delete lblEstado;
	delete cbEstados;
	delete lblTiempoVisible;
	delete nmTiempoVisible;
	delete pdlg;
	delete tabSPanel;

}

ZGui* zBT_Panel::getParent()
{
	return parent;
}

void zBT_Panel::slotLanguageChanged(bool fRedraw)
{
	ZComboBox* cbEstados;

	getlblEstado()->setText( lng->getText("BT_STATE") );

	getlblTiempoVisible()->setText( lng->getText("BT_VISIBLE_TIME") );

	cbEstados = getcbEstados();
	cbEstados->clear();
		cbEstados->insertItem(lng->getText("BT_INACTIVE"),0);
		cbEstados->insertItem(lng->getText("BT_INVISIBLE"), 1);
		cbEstados->insertItem(lng->getText("BT_VISIBLE"), 2);
	cbEstados->setCurrentItem( UTIL_GetBtStatus() );
	
	getlblTiempoUnidad()->setText(lng->getText("BT_MINUTES") );

}

/*
	SINGLETON PART
*/

ZLabel* zBT_Panel::getlblEstado()
{
	if (lblEstado!=NULL)
		return lblEstado;
	
	lblEstado = new ZLabel( "", parent, "lblEstado", 0, (ZSkinService::WidgetClsID)4);
		lblEstado->setFixedWidth(240);
		lblEstado->setAutoResize(true);
		lblEstado->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );
		
	return lblEstado;
}

ZComboBox* zBT_Panel::getcbEstados()
{
	if (cbEstados!=NULL)
		return cbEstados;
		
	cbEstados = new ZComboBox(parent,"cbEstados",true);
		cbEstados->setFixedWidth(220);
		
		cbEstados->setCurrentItem( UTIL_GetBtStatus() );
	return cbEstados;
}

ZScrollPanel* zBT_Panel::getSPanel()
{
	if (tabSPanel!=NULL)
		return tabSPanel;
		
	tabSPanel = new ZScrollPanel(this, "ZScrollPanel_Language", 0, (ZSkinService::WidgetClsID)4);
	
	return tabSPanel;
}

ZLabel* zBT_Panel::getlblMensaje()
{
	if (lblMensaje!=NULL)
		return lblMensaje;
	
	lblMensaje = new ZLabel("Para usar esta funcionalidad " \
							"el Bluetooth debe de haber sido activado antes.",\
							parent, "lblMensaje", 0, (ZSkinService::WidgetClsID)4);
		lblMensaje->setFixedWidth(220);
		lblMensaje->setAutoResize(true);

	lblMensaje->setFontColor( QColor(220,0,0) );

	return lblMensaje;
}

ZLabel* zBT_Panel::getlblTiempoVisible()
{
	if (lblTiempoVisible!=NULL)
		return lblTiempoVisible;

	lblTiempoVisible = new ZLabel( "", parent, "lblTiempoVisible", 0, (ZSkinService::WidgetClsID)4);
		lblTiempoVisible->setFixedWidth(80);
		lblTiempoVisible->setAutoResize(true);
		lblTiempoVisible->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );

	return lblTiempoVisible;
}

ZNumModule* zBT_Panel::getnmTiempoVisible()
{
	if (nmTiempoVisible!=NULL)
		return nmTiempoVisible;
		
	// NumPicker for the value of the option
	nmTiempoVisible = new ZNumModule(2, parent, "nmTiempoVisible", 0, (ZSkinService::WidgetClsID)4);
		nmTiempoVisible->setFixedWidth(150);
		nmTiempoVisible->setFixedHeight(80);

		// To avoid the annoying comportament while it has the focus and change it's value insteand of going to the previous Control
		nmTiempoVisible->setFocusProxy(parent);

		nmTiempoVisible->setLabel( getlblTiempoUnidad(), (ZNumModule::RelativeAlignment)1); 

		nmTiempoVisible->setMinValue(1);
		nmTiempoVisible->setMaxValue(10);
	return nmTiempoVisible;
}

ZLabel* zBT_Panel::getlblTiempoUnidad()
{
	if (lblTiempoUnidad!=NULL)
		return lblTiempoUnidad;

	// Label for indicating the units (minutes)
	lblTiempoUnidad = new ZLabel("",nmTiempoVisible,"lblTiempoUnidad", 0, (ZSkinService::WidgetClsID) 4);
		lblTiempoUnidad->setFixedWidth(100);
		lblTiempoUnidad->setAutoResize(true);

	return lblTiempoUnidad;
}

void zBT_Panel::showBTWaiting(QString title,QString message)
{
	bt_msg = NULL;
	FD_ZERO(&readSet);
	FD_SET(socketFd, &readSet);

	if (pdlg)
		delete pdlg;			
	pdlg = new ZProgressDlg(title,  "", 100, 220);

	pdlg->setInstructText(message);
	if (pdlg)
	{
		pdlg->setAutoClose(false);
		pdlg->setProgress(0);
		pdlg->setProgressTotalStep(10);
		pdlg->show();
		QTimer::singleShot(100, this, SLOT(slotProgressDlgTimer()));
		connect(pdlg, SIGNAL(cancelled()), this, SLOT(slotProgressDlgCancelled()));
	}
}
/*
	SLOTS
*/
void zBT_Panel::timerEvent(QTimerEvent* e)
{
	if (getcbEstados()->count() == 0 & currentEstado == -1 )
		return;
	else if ( currentEstado == -1 )
		currentEstado = getcbEstados()->currentItem();

	int selectEstado = getcbEstados()->currentItem();
	MBT_DISCOVERABLE_MODE_T mode;

	int prevTime = cfg_bt->readNumEntry("Bluetooth Setup", "DiscoverablePeriod", 1);
	if ( prevTime!=getnmTiempoVisible()->getValue() )
		cfg_bt->writeEntry("Bluetooth Setup", "DiscoverablePeriod", getnmTiempoVisible()->getValue() );

	if (selectEstado == currentEstado)
		return;

	switch (selectEstado)
	{
		case 0:
			cout<<" BT Turn it OFF ..."<<endl;
			getcbEstados()->setEnabled(false);
			MBTAPI_CTRL_PowerOff();
			break;
		case 1:
			// Turn it ON
			cout<<" BT Turn it ON and NON-DISCOVERABLE mode ..."<<endl;
			MBTAPI_CTRL_GetDiscoverableMode(&mode);
			if (mode!=MBT_DISCOVERABLE_MODE_OFF)
			{
				MBTAPI_CTRL_SetDiscoverableMode(MBT_DISCOVERABLE_MODE_OFF, 0);
				showBTWaiting( lng->getText("BT_WAITING"), lng->getText("BT_SETTING_NONDISCOVERABLE") );
			}
			break;
		case 2:
			// Turn it ON and set to Discoverable
			cout<<" BT Turn it ON and DISCOVERABLE mode ..."<<endl;
            MBTAPI_CTRL_SetDiscoverableMode(MBT_DISCOVERABLE_MODE_GIAC, cfg_bt->readNumEntry("Bluetooth Setup", "DiscoverablePeriod", 1) * 60 );
			showBTWaiting( lng->getText("BT_WAITING"), lng->getText("BT_SETTING_DISCOVERABLE") );
			break;
	}
	currentEstado = selectEstado;
}

static const char *discoverableModeNames[] = 
{
    "Non-discoverable mode",
    "General discoverable mode",
    "Limited discoverable mode"
};

bool HandleMessage(MBT_MESSAGE_HEADER_T *msg)
{

    MBT_CTRL_POWER_STATUS_IND_T *powerStatusInd;
//    MBT_CTRL_SET_LOCAL_COD_CFM_T *codCfm;
    MBT_CTRL_SET_DISCOVERABLE_MODE_CFM_T *discoverableCfm;

    MBT_CTRL_DISCOVERABLE_STATUS_IND_T *discoverableInd;

    switch (msg->primitive)
   	{
        case MBT_MSG_CTRL_POWER_STATUS_IND:
            fprintf(stderr,"\nReceive the MBT_MSG_CTRL_POWER_STATUS_IND message\n");
            powerStatusInd = (MBT_CTRL_POWER_STATUS_IND_T *)msg;
            {
				//handlePowerStatusInd((MBT_CTRL_POWER_STATUS_IND_T *)msg);
                //exit(0);
            }
            return true;

		case MBT_MSG_CTRL_SET_DISCOVERABLE_MODE_CFM:
            discoverableCfm = (MBT_CTRL_SET_DISCOVERABLE_MODE_CFM_T *)msg;
            if (discoverableCfm->result != MBT_CTRL_SET_DISCOVERABLE_OK)
            {
                fprintf(stderr,"Failed to set %s!\n", discoverableModeNames[discoverableCfm->mode]);
            }
            return false;
			
		case MBT_MSG_CTRL_DISCOVERABLE_STATUS_IND:
            fprintf(stderr,"\nReceive the MBT_MSG_CTRL_DISCOVERABLE_STATUS_IND message\n");
			discoverableInd = (MBT_CTRL_DISCOVERABLE_STATUS_IND_T *)msg;
            fprintf(stderr,"The discoverable mode is changed to %s!\n", discoverableModeNames[discoverableInd->mode]);
            //handleDiscoverableStatusInd((MBT_CTRL_DISCOVERABLE_STATUS_IND_T *)msg);
            return true;

        default:
            break;
    }
	return false;
}


void zBT_Panel::slotProgressDlgTimer()
{
	bool bt_Done=false;
	if ((pdlg->getProgress() < pdlg->getProgressTotalStep()) && pdlg->isVisible())
	{
		if ((select(socketFd + 1, &readSet, NULL, NULL, NULL) > 0) && FD_ISSET(socketFd, &readSet))
        {
            if (MBTAPI_ReadAndMallocMsg(socketFd, &bt_msg) != MBT_RESULT_SUCCESS)
            {
                fprintf(stderr,"Error messages from socket, quit...\n");

            }else{
	            fprintf(stderr,"Receiving the message, id = 0x%04X.\n", bt_msg->primitive);

	            // Handle messages
	            bt_Done = HandleMessage(bt_msg);

	            free(bt_msg);
	            bt_msg = NULL;
			}
        }

		if (!bt_Done)
		{
			pdlg->setProgress(pdlg->getProgress() + 1);
			QTimer::singleShot(4 * 10, this, SLOT( slotProgressDlgTimer() ) );
		}else{
			pdlg->cancel();
		}		
	}else if (pdlg->getProgress() < pdlg->getProgressTotalStep() )
	{
		pdlg->cancel();
	}
	return;
}
void zBT_Panel::slotProgressDlgCancelled()
{
}
