/****************************************************************************
* e2Ligth4 - Light Control  for Rokr E2
*
* Autor: Carlos Aguilar aka "Kidscracker" kidscracker@gmail.com
*
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//#define EZX_H

#include "ZGui.h"
#include "zSound_Panel.h"

#define EZX_VOLUMETABLE_CFG "/usr/SYSqtapp/sound/ezx_volumetable.cfg"
#define MY_FAVOR_EZX_VOLUMETABLE_CFG "/ezxlocal/download/mystuff/unKnown/My_FAVOR/ezx_volumetable.cfg"

extern ZConfig *ezx_system;
extern LanguageConfig *lng;

using namespace std;

GEQ_View::GEQ_View(QWidget * parent,const char * name, WFlags f,const ZSkinService::WidgetClsID clsId)
             :ZWidget(parent,name,f,clsId)
{
    init(clsId);
}

GEQ_View::GEQ_View(QPixmap & pixmap,QWidget * parent,const char * name, WFlags f,const ZSkinService::WidgetClsID clsId)
              :ZWidget(parent,name,f,clsId)
{
    init(clsId);
    setPixmap(pixmap);
}


GEQ_View::~GEQ_View()
{    
    clearContents();
}

void GEQ_View::setPixmap(const QPixmap & pixmap)
{
    if ( !lpixmap || lpixmap->serialNumber() != pixmap.serialNumber() ) 
    {
	    clearContents();
	    lpixmap = new QPixmap( pixmap );
    }

    if ( lpixmap->depth() == 1 && !lpixmap->mask() )
	    lpixmap->setMask( *((QBitmap *)lpixmap) );
    
//    int w = lpixmap->width(), h = lpixmap->height();

    //ZSkinTool tool;
//    QPixmap pp = *lpixmap;
/*
    if( pp.width() > w )
      pp = tool.process(pp,w,pp.height(),spScale);

    if( pp.height() > h )
      pp = tool.process(pp,pp.width(),h,spScale);
*/
    QWidget::resize( sizeHint() );
}

void GEQ_View::init(const ZSkinService::WidgetClsID clsId)
{
    updateSkin(FALSE);
    lpixmap = 0;
    setBackgroundMode( NoBackground );
    QWidget::resize( sizeHint() );
}

QRect GEQ_View::getContentRect()
{
	return QRect(0,0,width(), height());
}


void GEQ_View::paintEvent(QPaintEvent * )
{
    QRect cr = getContentRect();
    
    QPixmap pm(width(), height());

    QPainter p;

    p.begin(&pm, this);

    pm.fill(this, QPoint(0,0));
    

    drawBorder(p);
    drawBackground(p);

	if(lpixmap)
	{
		int xm = cr.left() + ( cr.width() - lpixmap->width() )/2 ;
		int ym = cr.top() + ( cr.height() - lpixmap->height() )/2;
    
		p.drawPixmap(cr.left() + (cr.width() - lpixmap->width())/2,cr.top() + (cr.height() - lpixmap->height())/2,*lpixmap);

		QFont tmp = QFont(font());
		tmp.setPixelSize(10);
		QFontMetrics fm(tmp);
		int fheight = fm.height();
		int x = 0, y = 0;
		
		p.setFont( tmp );
		QString num;
		for (int i = 0; i<10; i++)
		{
			int valor = (*ptrCurrentVolumeTable)[indexCurrentVolumeTableIndex].values[i] ;
			num.setNum(valor);
			x = i*15 + 5 + xm;
			y = fheight - 1 + ym;
			y += ( 25 - valor )/2 ;
			p.setPen( QColor( (31+valor)*4,255,255,QColor::Hsv) );
			p.drawText( x , y, num, strlen(num.ascii()), QPainter::RTL);
		}

	}
    p.end();
    bitBlt(this, 0, 0, &pm, 0, 0);

}

void GEQ_View::clearContents()
{
    if(lpixmap)
        delete lpixmap;
    lpixmap = 0;
}

QSize GEQ_View::sizeHint() const
{
    int w = width();
    int h = height();
    if(lpixmap)
	{
		w = lpixmap->width();
		h = lpixmap->height();
	}
    return QSize(w,h);
}

void GEQ_View::setPalette(const QPalette &)
{
    updateSkin(TRUE);
}

void GEQ_View::setVolumeTableList(VolumeTable_EntryList* ptrVolumeTable)
{
	ptrCurrentVolumeTable = ptrVolumeTable;
}
void GEQ_View::setCurrentTable(int index)
{
	indexCurrentVolumeTableIndex = index;
}


/******/

zSound_Panel::zSound_Panel ( ZGui * _parent) : 
	zBase_Panel( (QWidget*)_parent, "tabPage_Sound")
{
	// Set parent QWidget (ZGui in this case)
	parent = _parent;
	tabSPanel=NULL;
/*
	nmTiempoAlarma = NULL;
	lblTiempoAlarma = NULL;
	lblTiempoUnidad = NULL;
*/
	tabVolModSPanel = NULL;
	lblVolModTitle = NULL;
	cbOpcion = NULL;
	lblFrecuencia = NULL;
	cbFrecuencia = NULL;
	lblFrecVolume = NULL;
	slmVolValue = NULL;
	geqView = NULL;
	
	lastOption = -1;
	lastFrecuency = -1;
	LoadVolumeTable();
	//getnmTiempoAlarma()->setValue( ezx_system->readNumEntry("ALARMCLOCK", "AlertTime", 1) );

	// Set ContentWidget
	setContentWidget( getVolModSPanel() );
	
	slotOpcionActivated(0);

//	setContentWidget( getSPanel() );

	return;
}

zSound_Panel::~zSound_Panel()
{
	killTimers();
	//SaveVolumeTable("/mmc/mmca1/tmp/ezx_volumetable.cfg");
	SaveVolumeTable(MY_FAVOR_EZX_VOLUMETABLE_CFG);
/*
	delete lblTiempoAlarma;
	delete nmTiempoAlarma;
	delete tabSPanel;
*/
}

ZGui* zSound_Panel::getParent()
{
	return parent;
}

void zSound_Panel::slotLanguageChanged(bool fRedraw)
{
	if (tabVolModSPanel)
	{
	/*
		getlblTiempoAlarma()->setText(lng->getText("ALARM_TIME"));
		getlblTiempoUnidad()->setText(lng->getText("BT_MINUTES"));

		getlblTiempoAlarma()->setText( lng->getText("KEYPAD_BACKLIGHT") );
		getlblTiempoUnidad()->setText( lng->getText("SECONDS") );
	*/	
		getlblVolModTitle()->setText( lng->getText("SND_VOL_MOD_TITLE") );
		getlblFrecuencia()->setText( lng->getText("SND_FREQUENCY") );
		
		GainMessage = new QString( lng->getText("SND_GAIN_%1_DB") );
		
		getlblFrecVolume()->setText( GainMessage->arg( (*ptrVolumeTable)[lastOption].values[lastFrecuency] ) );
	}
}

void zSound_Panel::LoadVolumeTable()
{
    QStringList lines;
	QStringList values;
	char cmdBuff[512];
	ptrVolumeTable = new VolumeTable_EntryList();
	
	
	if ( !QFile(MY_FAVOR_EZX_VOLUMETABLE_CFG).exists() )
	{
		cout<<"Copying "<<MY_FAVOR_EZX_VOLUMETABLE_CFG<<" ... ";
		

		sprintf(cmdBuff,"cp %s %s", EZX_VOLUMETABLE_CFG, MY_FAVOR_EZX_VOLUMETABLE_CFG);
		system(cmdBuff);

		if ( !QFile(MY_FAVOR_EZX_VOLUMETABLE_CFG).exists() )
		{
			cout<<"[ERROR]"<<endl;
			cout<<"ERROR while copying "<<EZX_VOLUMETABLE_CFG<<" to "<<MY_FAVOR_EZX_VOLUMETABLE_CFG<<endl;
			exit(-1);
		}
		cout<<"[DONE]"<<endl;
	}

	QFile file(MY_FAVOR_EZX_VOLUMETABLE_CFG);
	cout<<"SoundManager using "<<MY_FAVOR_EZX_VOLUMETABLE_CFG<<endl;

    if ( file.open( IO_ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;
	
        while ( !stream.atEnd() )
		{
            line = stream.readLine();
			if ( !stream.atEnd() )
			{
				VolumeTable_Entry entry;;
				entry.Name = new QString(line.mid( 1, strlen( line.ascii() )-2 ) );
				line = stream.readLine();
				int line_length = strlen( line.ascii() );
				if ( line.right(1).compare(";") == 0 )
					line = line.mid(0,line_length-1);

				values = QStringList::split(",", line);
				int index = 0;
		        for ( QStringList::Iterator it = values.begin(); it != values.end(); ++it )
					entry.values[index++] = QString(*it).toInt();
				(*ptrVolumeTable)+=entry;
			}
		}
		file.close();
    }
}

void zSound_Panel::SaveVolumeTable(QString name)
{
    QFile file( name );

    if ( file.open( IO_WriteOnly | IO_Truncate ) ) {
        QTextStream stream( &file );
        QString line;
	
		for ( VolumeTable_EntryList::Iterator it = ptrVolumeTable->begin(); it != ptrVolumeTable->end(); ++it )
		{
			stream<<"["<<(* (*it).Name)<<"]"<<endl;
			for (int i=0;i<9;i++)
				stream<<(*it).values[i]<<", ";
			stream<< (*it).values[9] <<";"<<endl;
		}
		stream<<"[end]"<<endl;
		file.close();
    }
}


/*
	SINGLETON PART
*/
ZScrollPanel* zSound_Panel::getSPanel()
{
	if (tabSPanel!=NULL)
		return tabSPanel;
		
	tabSPanel = new ZScrollPanel(this, "ZScrollPanel_Sound", 0, (ZSkinService::WidgetClsID)4);

	ZPressButton* pbVolumeTableEqualizer = new ZPressButton( "Modify Volume Table", this, "ZPressButton_pbVolumeTableEqualizer");
	tabSPanel->addChild( pbVolumeTableEqualizer, 10,20);
	//ZPressButton* pbVolume = new ZPressButton( "Modify Volume Table", this, "ZPressButton_pbVolume");	
	//tabSPanel->addChild( pbVolume, 10,70);
	//connect( pbVolumeTableEqualizer, SIGNAL( selectionStateChanged (ZPressButton::SelectionState) ), this, SLOT( slotModificarVolumeTable(ZPressButton::SelectionState) ) );
	//	connect( getlstIdioma() , SIGNAL( selected (int) ), this, SLOT( slotLanguageListSelected(int) ) );

//	panel->resizeContents(240,320);
	
/*
	tabSPanel->addChild( getlblTiempoAlarma(), 0, 60);
	tabSPanel->addChild( getnmTiempoAlarma(), 90, 40);
*/
/*
    ZScrollView* sv = new ZScrollView( this, "ZScrollView", 0) ;
    QVBox* big_box = new QVBox(sv->viewport());
    sv->addChild(big_box);
	sv->resize(220,120);
*/
	//tabSPanel->addChild(sv->viewport(), 10, 130);
/*
	ZScrollPanel* pp = new ZScrollPanel(tabSPanel, "ZScrollPanel_Sound", 0, (ZSkinService::WidgetClsID)3);
	pp->setStaticBackground(true);
	pp->setHScrollBarMode(ZScrollPanel::AlwaysOff);
	pp->setVScrollBarMode(ZScrollPanel::Auto);

	pp->setFixedWidth(220);
	pp->setFixedHeight(120);
*/
/*
	ZSliderModule* slm1 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue1", (ZSkinService::WidgetClsID) 4);
		slm1->setFocusProxy(this);
	ZSliderModule* slm2 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue2", (ZSkinService::WidgetClsID) 4);
		slm2->setFocusProxy(this);
	ZSliderModule* slm3 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue3", (ZSkinService::WidgetClsID) 4);
		slm3->setFocusProxy(this);
	ZSliderModule* slm4 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue4", (ZSkinService::WidgetClsID) 4);
		slm4->setFocusProxy(this);
	ZSliderModule* slm5 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue5", (ZSkinService::WidgetClsID) 4);
		slm5->setFocusProxy(this);
	ZSliderModule* slm6 = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, big_box, "ZSliderModule_VolValue5", (ZSkinService::WidgetClsID) 4);
		slm6->setFocusProxy(this);
	
	sv->addChild( slm1, 0, 0);
	sv->addChild( slm2, 0, 30);
	sv->addChild( slm3, 0, 60);
	sv->addChild( slm4, 0, 90);
	sv->addChild( slm5, 0, 120);
	sv->addChild( slm6, 0, 150);
*/
/*
	cout<<"contentsHeight() : "<<pp->contentsHeight()<<endl;
	cout<<"visibleHeight() : "<<pp->visibleHeight()<<endl;
*/
	return tabSPanel;
}

ZScrollPanel* zSound_Panel::getVolModSPanel()
{
	if (tabVolModSPanel!=NULL)
		return tabVolModSPanel;

	tabVolModSPanel = new ZScrollPanel(this, "ZScrollPanel_Sound_VolMod", 0, (ZSkinService::WidgetClsID)4);
		tabVolModSPanel->addChild( getlblVolModTitle(), 5, 5);
		tabVolModSPanel->addChild( getcbOpcion(), 5, 40);
		tabVolModSPanel->addChild( getlblFrecuencia(), 5, 80);
		tabVolModSPanel->addChild( getcbFrecuencia(), 140, 80);
		tabVolModSPanel->addChild( getlblFrecVolume(), 0, 120);
		tabVolModSPanel->addChild( getslmVolValue(), 5, 150);
		tabVolModSPanel->addChild( getgeqView(), 0, 175);

	return tabVolModSPanel;
}
/*
ZLabel* zSound_Panel::getlblTiempoAlarma()
{
	if (lblTiempoAlarma!=NULL)
		return lblTiempoAlarma;

	lblTiempoAlarma = new ZLabel( "", tabSPanel, "lblTiempoAlarma", 0, (ZSkinService::WidgetClsID)4);
		lblTiempoAlarma->setFixedWidth(80);
		lblTiempoAlarma->setAutoResize(true);
		lblTiempoAlarma->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );

	return lblTiempoAlarma;
}

ZNumModule* zSound_Panel::getnmTiempoAlarma()
{
	if (nmTiempoAlarma!=NULL)
		return nmTiempoAlarma;
		
	// NumPicker for the value of the option
	nmTiempoAlarma = new ZNumModule(2, tabSPanel, "nmTiempoAlarma", 0, (ZSkinService::WidgetClsID)4);
		nmTiempoAlarma->setFixedWidth(150);
		nmTiempoAlarma->setFixedHeight(80);

		// To avoid the annoying comportament while it has the focus and change it's value insteand of going to the previous Control
		nmTiempoAlarma->setFocusProxy(parent);

		nmTiempoAlarma->setLabel( getlblTiempoUnidad(), (ZNumModule::RelativeAlignment)1); 

		nmTiempoAlarma->setMinValue(1);
		nmTiempoAlarma->setMaxValue(10);
	return nmTiempoAlarma;
}

ZLabel* zSound_Panel::getlblTiempoUnidad()
{
	if (lblTiempoUnidad!=NULL)
		return lblTiempoUnidad;

	// Label for indicating the units (minutes)
	lblTiempoUnidad = new ZLabel("",nmTiempoAlarma,"lblTiempoUnidad", 0, (ZSkinService::WidgetClsID) 4);
		lblTiempoUnidad->setFixedWidth(100);
		lblTiempoUnidad->setAutoResize(true);

	return lblTiempoUnidad;
}
*/
ZLabel* zSound_Panel::getlblVolModTitle()
{
	if (lblVolModTitle!=NULL)
		return lblVolModTitle;

	lblVolModTitle = new ZLabel("Editor de VolumeTable",lblVolModTitle,"lblVolModTitle", 0, (ZSkinService::WidgetClsID) 4);
		lblVolModTitle->setFixedWidth(240);
		lblVolModTitle->setAutoResize(true);
		lblVolModTitle->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );

	lblVolModTitle->setFontColor( QColor(0,0,200) );
	
	return lblVolModTitle;
}

ZComboBox* zSound_Panel::getcbOpcion()
{
	if (cbOpcion!=NULL)
		return cbOpcion;
		
	cbOpcion = new ZComboBox(tabVolModSPanel,"cbOpcion",true);
	cbOpcion->setFixedWidth(220);
	int index=0;
	for ( VolumeTable_EntryList::Iterator it = ptrVolumeTable->begin(); it != ptrVolumeTable->end(); ++it )
	{
		cbOpcion->insertItem((* (*it).Name), index++);
	}
	
	cbOpcion->setCurrentItem(0);
	
	lastOption = 0;
	
	connect(cbOpcion, SIGNAL(activated(int)), this, SLOT(slotOpcionActivated(int)) );
	
	return cbOpcion;
}

ZLabel* zSound_Panel::getlblFrecuencia()
{
	if (lblFrecuencia!=NULL)
		return lblFrecuencia;

	lblFrecuencia = new ZLabel("Frecuencia",lblFrecuencia,"lblFrecuencia", 0, (ZSkinService::WidgetClsID) 4);
		lblFrecuencia->setFixedWidth(135);
		lblFrecuencia->setAutoResize(true);
		lblFrecuencia->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );

	return lblFrecuencia;
}

ZComboBox* zSound_Panel::getcbFrecuencia()
{
	if (cbFrecuencia!=NULL)
		return cbFrecuencia;
		
	cbFrecuencia = new ZComboBox(tabVolModSPanel,"cbFrecuencia",true);
	cbFrecuencia->setFixedWidth(85);
		cbFrecuencia->insertItem("31 Hz.", 0);
		cbFrecuencia->insertItem("62 Hz.", 1);
		cbFrecuencia->insertItem("125 Hz.", 2);
		cbFrecuencia->insertItem("250 Hz.", 3);
		cbFrecuencia->insertItem("500 Hz.", 4);
		cbFrecuencia->insertItem("1 kHz.", 5);
		cbFrecuencia->insertItem("2 kHz.", 6);
		cbFrecuencia->insertItem("4 kHz.", 7);
		cbFrecuencia->insertItem("8 kHz.", 8);
		cbFrecuencia->insertItem("16 kHz.", 9);
		
	cbFrecuencia->setCurrentItem(0);
	lastFrecuency = 0;
	
	connect(cbFrecuencia, SIGNAL(activated(int)), this, SLOT(slotFrecuenciaActivated(int)) );
	
	return cbFrecuencia;
}

ZLabel* zSound_Panel::getlblFrecVolume()
{
	if (lblFrecVolume!=NULL)
		return lblFrecVolume;

	lblFrecVolume = new ZLabel("",tabVolModSPanel,"lblFrecVolume", 0, (ZSkinService::WidgetClsID) 4);
		lblFrecVolume->setFixedWidth(240);
		lblFrecVolume->setAutoResize(true);
		lblFrecVolume->setAlignment( (ZLabel::AlignmentFlags) (Qt::AlignVCenter | Qt::AlignCenter ) );
	GainMessage = new QString("%1");
	return lblFrecVolume;
}

ZSliderModule* zSound_Panel::getslmVolValue()
{
	if (slmVolValue!=NULL)
		return slmVolValue;

	slmVolValue = new ZSliderModule( -25, 25, 1, 0, ZSlider::NULL_MINMAX, tabVolModSPanel->viewport(), "ZSliderModule_VolValue", (ZSkinService::WidgetClsID) 4);
		slmVolValue->setFocusProxy(this);
	    int sliderwidth = 240 - 10;
		slmVolValue->setThickness(sliderwidth);
	
	connect(slmVolValue->getSlider(), SIGNAL(valueChanged(int)), this, SLOT(slotValueChanged(int)) );

	return slmVolValue;
}

GEQ_View* zSound_Panel::getgeqView()
{
	if (geqView)
		return geqView;

	geqView = new GEQ_View(tabVolModSPanel->viewport(),"geqView", 0, (ZSkinService::WidgetClsID)4);
		geqView->setPixmap( QPixmap(parent->getAppDir().absFilePath("img/geq_bg.png")) );
		geqView->resize(240,40);
		geqView->setVolumeTableList(ptrVolumeTable);
		geqView->setCurrentTable(0);
	return geqView;
}

/*
	SLOTS
*/
void zSound_Panel::timerEvent(QTimerEvent* e)
{
}

void zSound_Panel::slotValueChanged(int value)
{
    QString lblSlider = GainMessage->arg(value);
	lastOption = cbOpcion->currentItem();
	lastFrecuency = cbFrecuencia->currentItem();
    getlblFrecVolume()->setText(lblSlider);
	(*ptrVolumeTable)[lastOption].values[lastFrecuency] = value;
	getgeqView()->update();
	
}

void zSound_Panel::slotOpcionActivated(int index)
{
	lastOption = index;
	geqView->setCurrentTable(index);
	int valor = (*ptrVolumeTable)[lastOption].values[lastFrecuency];
	slmVolValue->getSlider()->setValue(valor);
}

void zSound_Panel::slotFrecuenciaActivated(int index)
{
	lastFrecuency = index;
	int valor = (*ptrVolumeTable)[lastOption].values[lastFrecuency];
	slmVolValue->getSlider()->setValue(valor);
}
